/*
 * dfb_api.c - api to dfb interface
 */

/* std includes */
#include <stdio.h>
#include <string.h>
#include <unistd.h>

/* dfb includes */
#include <directfb.h>

#define MY_FONT_HEIGHT 18

/* global stuff */

/* well ... */
IDirectFB *dfb = NULL;

IDirectFBSurface *primary = NULL;
IDirectFBSurface *logo = NULL;
IDirectFBSurface *video = NULL;
IDirectFBSurface *window = NULL;

IDirectFBImageProvider *image_provider = NULL;
IDirectFBVideoProvider *video_provider = NULL;

int screen_width  = 0;
int screen_height = 0;

char dfb_video_dev[]=VIDEO_DEV;
char text_top[]="Internet Video / Audio Conferencing";
char dfb_image[]="./images/ivac_logo.png";
char dfb_font[]="./fonts/decker.ttf";

/* small api - directfb usage kinda sux: foo->bar(foo,...) wtf?!?!?! */

int dfb_init(int *argc,char **argv) {
 DirectFBInit(argc,&argv);
 DirectFBCreate(&dfb);
 dfb->SetCooperativeLevel(dfb,DFSCL_FULLSCREEN);
 return 1;
}

int check_accel(void) {
 DFBCardCapabilities caps;
 dfb->GetCardCapabilities(dfb,&caps);
 return (caps.acceleration_mask);
}

int create_primary_surface(void) {
 DFBSurfaceDescription desc;
 memset(&desc,0,sizeof(DFBSurfaceDescription));
 desc.flags=DSDESC_CAPS;
 desc.caps=DSCAPS_PRIMARY;
 dfb->CreateSurface(dfb,&desc,&primary);
 return 1;
}

int get_surface_size(IDirectFBSurface *surface,int *width,int *height) {
 surface->GetSize(surface,width,height);
 return 1;
}

int get_primary_surface_size(void) {
 get_surface_size(primary,&screen_width,&screen_height);
 return 1;
}

int create_logo_surface(void) {
 DFBSurfaceDescription desc;
 DFBImageDescription image_desc;
 dfb->CreateImageProvider(dfb,dfb_image,&image_provider);
 image_provider->GetSurfaceDescription(image_provider,&desc);
 image_provider->GetImageDescription(image_provider,&image_desc);
 printf("image description: caps=%x red=%hhd green=%hhd blue=%hhd\n",image_desc.caps,
							image_desc.colorkey_r,
							image_desc.colorkey_g,
							image_desc.colorkey_b);
 dfb->CreateSurface(dfb,&desc,&logo);
 image_provider->RenderTo(image_provider,logo,NULL);
 image_provider->Release(image_provider);
 return 1;
}
 
int create_video_surface(void) {
 DFBSurfaceDescription desc;
 DFBVideoProviderCapabilities caps;
 dfb->CreateVideoProvider(dfb,dfb_video_dev,&video_provider);
 video_provider->GetSurfaceDescription(video_provider,&desc);
 video_provider->GetCapabilities(video_provider,&caps);
 printf("video capabilities: %x\n",caps);
 dfb->CreateSurface(dfb,&desc,&video);
 video_provider->PlayTo(video_provider,video,NULL,NULL,NULL);
 video_provider->Release(video_provider);
 return 1;
}

int clear_screen(IDirectFBSurface *surface) {
 surface->FillRectangle(surface,0,0,screen_width,screen_height);
 return 1;
}

int release_surface(IDirectFBSurface *surface) {
 surface->Release(surface);
 return 1;
}

int release_dfb(void) {
 dfb->Release(dfb);
 return 1;
}

int blit_surface(IDirectFBSurface *source,int x,int y) {
 primary->Blit(primary,source,NULL,x,y);
 return 1;
}

int blit_logo(void) {
 int x,y;
 get_surface_size(logo,&x,&y);
 printf("size of logo width/height: %d/%d\n",x,y);
 blit_surface(logo,((screen_width-x)/2),((screen_height-y)/2));
 return 1;
}

int blit_video(void) {
 int x,y;
 get_surface_size(video,&x,&y);
 blit_surface(video,((screen_width-x)/2),((screen_height-y)/2));
 return 1;
}

int flip_it(void) {
 primary->Flip(primary,NULL,DSFLIP_WAITFORSYNC);
 return 1;
}

/* test api app */
int main (int argc, char **argv) {

 dfb_init(&argc,argv);

 check_accel();

 create_primary_surface();

 // create_video_surface();
 create_logo_surface();

 get_primary_surface_size();
 printf("primary surface width/height: %d/%d\n",screen_width,screen_height);

 clear_screen(primary);

 blit_logo();

 flip_it();

 sleep(10);

 release_surface(logo);

 release_surface(primary);

 release_dfb();

 return 1;
}
